#ifndef __ROCCAT_HSCALE_H__
#define __ROCCAT_HSCALE_H__

/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * roccat_hscale is a hscale coupled with a spinbutton on the right side
 */

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define ROCCAT_HSCALE_TYPE (roccat_hscale_get_type())
#define ROCCAT_HSCALE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), ROCCAT_HSCALE_TYPE, RoccatHScale))
#define IS_ROCCAT_HSCALE(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), ROCCAT_HSCALE_TYPE))

typedef struct _RoccatHScale RoccatHScale;
typedef struct _RoccatHScalePrivate RoccatHScalePrivate;

struct _RoccatHScale {
	GtkHBox hbox;
	RoccatHScalePrivate *priv;
};

GType roccat_hscale_get_type(void);
GtkWidget *roccat_hscale_new_with_range(gdouble min, gdouble max, gdouble step);

gdouble roccat_hscale_get_value(RoccatHScale *hscale);
void roccat_hscale_set_value_blocked(RoccatHScale *hscale, gdouble value);

void roccat_hscale_add_mark(RoccatHScale *hscale, gdouble value, GtkPositionType position, gchar const *markup);
void roccat_hscale_clear_marks(RoccatHScale *hscale);

guint roccat_hscale_get_digits(RoccatHScale *hscale);
void roccat_hscale_set_digits(RoccatHScale *hscale, guint digits);

void roccat_hscale_set_range(RoccatHScale *hscale, gdouble min, gdouble max);
void roccat_hscale_set_increment(RoccatHScale *hscale, gdouble step);

void roccat_hscale_set_draw_spin(RoccatHScale *hscale, gboolean draw_spin);

G_END_DECLS

#endif
